@@ -3,8 +3,8 @@
 import ipaddress
 import os
 import uuid
-from datetime import datetime, timedelta
-from typing import Tuple
+from collections.abc import Generator
+from datetime import datetime, timedelta, timezone
 
 import freezegun
 import pytest
@@ -17,8 +17,9 @@ from cryptography.x509.oid import NameOID
 from keycloak import KeycloakAdmin, KeycloakOpenID, KeycloakOpenIDConnection, KeycloakUMA
 
 
-class KeycloakTestEnv(object):
-    """Wrapper for test Keycloak connection configuration.
+class KeycloakTestEnv:
+    """
+    Wrapper for test Keycloak connection configuration.
 
     :param host: Hostname
     :type host: str
@@ -36,8 +37,9 @@ class KeycloakTestEnv(object):
         port: str = os.environ["KEYCLOAK_PORT"],
         username: str = os.environ["KEYCLOAK_ADMIN"],
         password: str = os.environ["KEYCLOAK_ADMIN_PASSWORD"],
-    ):
-        """Init method.
+    ) -> None:
+        """
+        Init method.
 
         :param host: Hostname
         :type host: str
@@ -48,87 +50,96 @@ class KeycloakTestEnv(object):
         :param password: Admin password
         :type password: str
         """
-        self.KEYCLOAK_HOST = host
-        self.KEYCLOAK_PORT = port
-        self.KEYCLOAK_ADMIN = username
-        self.KEYCLOAK_ADMIN_PASSWORD = password
+        self.keycloak_host = host
+        self.keycloak_port = port
+        self.keycloak_admin = username
+        self.keycloak_admin_password = password
 
     @property
-    def KEYCLOAK_HOST(self):
-        """Hostname getter.
+    def keycloak_host(self) -> str:
+        """
+        Hostname getter.
 
         :returns: Keycloak host
         :rtype: str
         """
-        return self._KEYCLOAK_HOST
+        return self._keycloak_host
 
-    @KEYCLOAK_HOST.setter
-    def KEYCLOAK_HOST(self, value: str):
-        """Hostname setter.
+    @keycloak_host.setter
+    def keycloak_host(self, value: str) -> None:
+        """
+        Hostname setter.
 
         :param value: Keycloak host
         :type value: str
         """
-        self._KEYCLOAK_HOST = value
+        self._keycloak_host = value
 
     @property
-    def KEYCLOAK_PORT(self):
-        """Port getter.
+    def keycloak_port(self) -> str:
+        """
+        Port getter.
 
         :returns: Keycloak port
         :rtype: str
         """
-        return self._KEYCLOAK_PORT
+        return self._keycloak_port
 
-    @KEYCLOAK_PORT.setter
-    def KEYCLOAK_PORT(self, value: str):
-        """Port setter.
+    @keycloak_port.setter
+    def keycloak_port(self, value: str) -> None:
+        """
+        Port setter.
 
         :param value: Keycloak port
         :type value: str
         """
-        self._KEYCLOAK_PORT = value
+        self._keycloak_port = value
 
     @property
-    def KEYCLOAK_ADMIN(self):
-        """Admin username getter.
+    def keycloak_admin(self) -> str:
+        """
+        Admin username getter.
 
         :returns: Admin username
         :rtype: str
         """
-        return self._KEYCLOAK_ADMIN
+        return self._keycloak_admin
 
-    @KEYCLOAK_ADMIN.setter
-    def KEYCLOAK_ADMIN(self, value: str):
-        """Admin username setter.
+    @keycloak_admin.setter
+    def keycloak_admin(self, value: str) -> None:
+        """
+        Admin username setter.
 
         :param value: Admin username
         :type value: str
         """
-        self._KEYCLOAK_ADMIN = value
+        self._keycloak_admin = value
 
     @property
-    def KEYCLOAK_ADMIN_PASSWORD(self):
-        """Admin password getter.
+    def keycloak_admin_password(self) -> str:
+        """
+        Admin password getter.
 
         :returns: Admin password
         :rtype: str
         """
-        return self._KEYCLOAK_ADMIN_PASSWORD
+        return self._keycloak_admin_password
 
-    @KEYCLOAK_ADMIN_PASSWORD.setter
-    def KEYCLOAK_ADMIN_PASSWORD(self, value: str):
-        """Admin password setter.
+    @keycloak_admin_password.setter
+    def keycloak_admin_password(self, value: str) -> None:
+        """
+        Admin password setter.
 
         :param value: Admin password
         :type value: str
         """
-        self._KEYCLOAK_ADMIN_PASSWORD = value
+        self._keycloak_admin_password = value
 
 
 @pytest.fixture
-def env():
-    """Fixture for getting the test environment configuration object.
+def env() -> KeycloakTestEnv:
+    """
+    Fixture for getting the test environment configuration object.
 
     :returns: Keycloak test environment object
     :rtype: KeycloakTestEnv
@@ -137,8 +148,9 @@ def env():
 
 
 @pytest.fixture
-def admin(env: KeycloakTestEnv):
-    """Fixture for initialized KeycloakAdmin class.
+def admin(env: KeycloakTestEnv) -> KeycloakAdmin:
+    """
+    Fixture for initialized KeycloakAdmin class.
 
     :param env: Keycloak test environment
     :type env: KeycloakTestEnv
@@ -146,16 +158,17 @@ def admin(env: KeycloakTestEnv):
     :rtype: KeycloakAdmin
     """
     return KeycloakAdmin(
-        server_url=f"http://{env.KEYCLOAK_HOST}:{env.KEYCLOAK_PORT}",
-        username=env.KEYCLOAK_ADMIN,
-        password=env.KEYCLOAK_ADMIN_PASSWORD,
+        server_url=f"http://{env.keycloak_host}:{env.keycloak_port}",
+        username=env.keycloak_admin,
+        password=env.keycloak_admin_password,
     )
 
 
 @pytest.fixture
 @freezegun.freeze_time("2023-02-25 10:00:00")
-def admin_frozen(env: KeycloakTestEnv):
-    """Fixture for initialized KeycloakAdmin class, with time frozen.
+def admin_frozen(env: KeycloakTestEnv) -> KeycloakAdmin:
+    """
+    Fixture for initialized KeycloakAdmin class, with time frozen.
 
     :param env: Keycloak test environment
     :type env: KeycloakTestEnv
@@ -163,15 +176,20 @@ def admin_frozen(env: KeycloakTestEnv):
     :rtype: KeycloakAdmin
     """
     return KeycloakAdmin(
-        server_url=f"http://{env.KEYCLOAK_HOST}:{env.KEYCLOAK_PORT}",
-        username=env.KEYCLOAK_ADMIN,
-        password=env.KEYCLOAK_ADMIN_PASSWORD,
+        server_url=f"http://{env.keycloak_host}:{env.keycloak_port}",
+        username=env.keycloak_admin,
+        password=env.keycloak_admin_password,
     )
 
 
 @pytest.fixture
-def oid(env: KeycloakTestEnv, realm: str, admin: KeycloakAdmin):
-    """Fixture for initialized KeycloakOpenID class.
+def oid(
+    env: KeycloakTestEnv,
+    realm: str,
+    admin: KeycloakAdmin,
+) -> Generator[KeycloakOpenID, None, None]:
+    """
+    Fixture for initialized KeycloakOpenID class.
 
     :param env: Keycloak test environment
     :type env: KeycloakTestEnv
@@ -193,11 +211,11 @@ def oid(env: KeycloakTestEnv, realm: str, admin: KeycloakAdmin):
             "enabled": True,
             "publicClient": True,
             "protocol": "openid-connect",
-        }
+        },
     )
     # Return OID
     yield KeycloakOpenID(
-        server_url=f"http://{env.KEYCLOAK_HOST}:{env.KEYCLOAK_PORT}",
+        server_url=f"http://{env.keycloak_host}:{env.keycloak_port}",
         realm_name=realm,
         client_id=client,
     )
@@ -206,8 +224,13 @@ def oid(env: KeycloakTestEnv, realm: str, admin: KeycloakAdmin):
 
 
 @pytest.fixture
-def oid_with_credentials(env: KeycloakTestEnv, realm: str, admin: KeycloakAdmin):
-    """Fixture for an initialized KeycloakOpenID class and a random user credentials.
+def oid_with_credentials(
+    env: KeycloakTestEnv,
+    realm: str,
+    admin: KeycloakAdmin,
+) -> Generator[tuple[KeycloakOpenID, str, str], None, None]:
+    """
+    Fixture for an initialized KeycloakOpenID class and a random user credentials.
 
     :param env: Keycloak test environment
     :type env: KeycloakTestEnv
@@ -232,7 +255,7 @@ def oid_with_credentials(env: KeycloakTestEnv, realm: str, admin: KeycloakAdmin)
             "protocol": "openid-connect",
             "secret": secret,
             "clientAuthenticatorType": "client-secret",
-        }
+        },
     )
     # Create user
     username = str(uuid.uuid4())
@@ -242,13 +265,17 @@ def oid_with_credentials(env: KeycloakTestEnv, realm: str, admin: KeycloakAdmin)
             "username": username,
             "email": f"{username}@test.test",
             "enabled": True,
-            "credentials": [{"type": "password", "value": password}],
-        }
+            "firstName": "first",
+            "lastName": "last",
+            "emailVerified": True,
+            "requiredActions": [],
+            "credentials": [{"type": "password", "value": password, "temporary": False}],
+        },
     )
 
     yield (
         KeycloakOpenID(
-            server_url=f"http://{env.KEYCLOAK_HOST}:{env.KEYCLOAK_PORT}",
+            server_url=f"http://{env.keycloak_host}:{env.keycloak_port}",
             realm_name=realm,
             client_id=client,
             client_secret_key=secret,
@@ -263,8 +290,13 @@ def oid_with_credentials(env: KeycloakTestEnv, realm: str, admin: KeycloakAdmin)
 
 
 @pytest.fixture
-def oid_with_credentials_authz(env: KeycloakTestEnv, realm: str, admin: KeycloakAdmin):
-    """Fixture for an initialized KeycloakOpenID class and a random user credentials.
+def oid_with_credentials_authz(
+    env: KeycloakTestEnv,
+    realm: str,
+    admin: KeycloakAdmin,
+) -> Generator[tuple[KeycloakOpenID, str, str], None, None]:
+    """
+    Fixture for an initialized KeycloakOpenID class and a random user credentials.
 
     :param env: Keycloak test environment
     :type env: KeycloakTestEnv
@@ -291,7 +323,7 @@ def oid_with_credentials_authz(env: KeycloakTestEnv, realm: str, admin: Keycloak
             "clientAuthenticatorType": "client-secret",
             "authorizationServicesEnabled": True,
             "serviceAccountsEnabled": True,
-        }
+        },
     )
     admin.create_client_authz_role_based_policy(
         client_id=client_id,
@@ -308,13 +340,17 @@ def oid_with_credentials_authz(env: KeycloakTestEnv, realm: str, admin: Keycloak
             "username": username,
             "email": f"{username}@test.test",
             "enabled": True,
-            "credentials": [{"type": "password", "value": password}],
-        }
+            "emailVerified": True,
+            "firstName": "first",
+            "lastName": "last",
+            "requiredActions": [],
+            "credentials": [{"type": "password", "value": password, "temporary": False}],
+        },
     )
 
     yield (
         KeycloakOpenID(
-            server_url=f"http://{env.KEYCLOAK_HOST}:{env.KEYCLOAK_PORT}",
+            server_url=f"http://{env.keycloak_host}:{env.keycloak_port}",
             realm_name=realm,
             client_id=client,
             client_secret_key=secret,
@@ -329,8 +365,76 @@ def oid_with_credentials_authz(env: KeycloakTestEnv, realm: str, admin: Keycloak
 
 
 @pytest.fixture
-def realm(admin: KeycloakAdmin) -> str:
-    """Fixture for a new random realm.
+def oid_with_credentials_device(
+    env: KeycloakTestEnv,
+    realm: str,
+    admin: KeycloakAdmin,
+) -> Generator[tuple[KeycloakOpenID, str, str], None, None]:
+    """
+    Fixture for an initialized KeycloakOpenID class and a random user credentials.
+
+    :param env: Keycloak test environment
+    :type env: KeycloakTestEnv
+    :param realm: Keycloak realm
+    :type realm: str
+    :param admin: Keycloak admin
+    :type admin: KeycloakAdmin
+    :yields: Keycloak OpenID client with user credentials
+    :rtype: Tuple[KeycloakOpenID, str, str]
+    """
+    # Set the realm
+    admin.change_current_realm(realm)
+    # Create client
+    client = str(uuid.uuid4())
+    secret = str(uuid.uuid4())
+    client_id = admin.create_client(
+        payload={
+            "name": client,
+            "clientId": client,
+            "enabled": True,
+            "publicClient": False,
+            "protocol": "openid-connect",
+            "secret": secret,
+            "clientAuthenticatorType": "client-secret",
+            "attributes": {"oauth2.device.authorization.grant.enabled": True},
+        },
+    )
+    # Create user
+    username = str(uuid.uuid4())
+    password = str(uuid.uuid4())
+    user_id = admin.create_user(
+        payload={
+            "username": username,
+            "email": f"{username}@test.test",
+            "enabled": True,
+            "firstName": "first",
+            "lastName": "last",
+            "emailVerified": True,
+            "requiredActions": [],
+            "credentials": [{"type": "password", "value": password, "temporary": False}],
+        },
+    )
+
+    yield (
+        KeycloakOpenID(
+            server_url=f"http://{env.keycloak_host}:{env.keycloak_port}",
+            realm_name=realm,
+            client_id=client,
+            client_secret_key=secret,
+        ),
+        username,
+        password,
+    )
+
+    # Cleanup
+    admin.delete_client(client_id=client_id)
+    admin.delete_user(user_id=user_id)
+
+
+@pytest.fixture
+def realm(admin: KeycloakAdmin) -> Generator[str, None, None]:
+    """
+    Fixture for a new random realm.
 
     :param admin: Keycloak admin
     :type admin: KeycloakAdmin
@@ -344,8 +448,9 @@ def realm(admin: KeycloakAdmin) -> str:
 
 
 @pytest.fixture
-def user(admin: KeycloakAdmin, realm: str) -> str:
-    """Fixture for a new random user.
+def user(admin: KeycloakAdmin, realm: str) -> Generator[str, None, None]:
+    """
+    Fixture for a new random user.
 
     :param admin: Keycloak admin
     :type admin: KeycloakAdmin
@@ -362,8 +467,9 @@ def user(admin: KeycloakAdmin, realm: str) -> str:
 
 
 @pytest.fixture
-def group(admin: KeycloakAdmin, realm: str) -> str:
-    """Fixture for a new random group.
+def group(admin: KeycloakAdmin, realm: str) -> Generator[str, None, None]:
+    """
+    Fixture for a new random group.
 
     :param admin: Keycloak admin
     :type admin: KeycloakAdmin
@@ -380,8 +486,9 @@ def group(admin: KeycloakAdmin, realm: str) -> str:
 
 
 @pytest.fixture
-def client(admin: KeycloakAdmin, realm: str) -> str:
-    """Fixture for a new random client.
+def client(admin: KeycloakAdmin, realm: str) -> Generator[str, None, None]:
+    """
+    Fixture for a new random client.
 
     :param admin: Keycloak admin
     :type admin: KeycloakAdmin
@@ -398,8 +505,9 @@ def client(admin: KeycloakAdmin, realm: str) -> str:
 
 
 @pytest.fixture
-def client_role(admin: KeycloakAdmin, realm: str, client: str) -> str:
-    """Fixture for a new random client role.
+def client_role(admin: KeycloakAdmin, realm: str, client: str) -> Generator[str, None, None]:
+    """
+    Fixture for a new random client role.
 
     :param admin: Keycloak admin
     :type admin: KeycloakAdmin
@@ -418,8 +526,14 @@ def client_role(admin: KeycloakAdmin, realm: str, client: str) -> str:
 
 
 @pytest.fixture
-def composite_client_role(admin: KeycloakAdmin, realm: str, client: str, client_role: str) -> str:
-    """Fixture for a new random composite client role.
+def composite_client_role(
+    admin: KeycloakAdmin,
+    realm: str,
+    client: str,
+    client_role: str,
+) -> Generator[str, None, None]:
+    """
+    Fixture for a new random composite client role.
 
     :param admin: Keycloak admin
     :type admin: KeycloakAdmin
@@ -442,8 +556,9 @@ def composite_client_role(admin: KeycloakAdmin, realm: str, client: str, client_
 
 
 @pytest.fixture
-def selfsigned_cert():
-    """Generate self signed certificate for a hostname, and optional IP addresses.
+def selfsigned_cert() -> tuple[str, str]:
+    """
+    Generate self signed certificate for a hostname, and optional IP addresses.
 
     :returns: Selfsigned certificate
     :rtype: Tuple[str, str]
@@ -454,7 +569,9 @@ def selfsigned_cert():
     # Generate our key
     if key is None:
         key = rsa.generate_private_key(
-            public_exponent=65537, key_size=2048, backend=default_backend()
+            public_exponent=65537,
+            key_size=2048,
+            backend=default_backend(),
         )
 
     name = x509.Name([x509.NameAttribute(NameOID.COMMON_NAME, hostname)])
@@ -473,7 +590,7 @@ def selfsigned_cert():
 
     # path_len=0 means this cert can only sign itself, not other certs.
     basic_contraints = x509.BasicConstraints(ca=True, path_length=0)
-    now = datetime.utcnow()
+    now = datetime.now(tz=timezone.utc)
     cert = (
         x509.CertificateBuilder()
         .subject_name(name)
@@ -497,8 +614,11 @@ def selfsigned_cert():
 
 
 @pytest.fixture
-def oid_connection_with_authz(oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str]):
-    """Fixture for initialized KeycloakUMA class.
+def oid_connection_with_authz(
+    oid_with_credentials_authz: tuple[KeycloakOpenID, str, str],
+) -> KeycloakOpenIDConnection:
+    """
+    Fixture for initialized KeycloakUMA class.
 
     :param oid_with_credentials_authz: Keycloak OpenID client with pre-configured user credentials
     :type oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str]
@@ -506,19 +626,19 @@ def oid_connection_with_authz(oid_with_credentials_authz: Tuple[KeycloakOpenID,
     :rtype: KeycloakOpenIDConnection
     """
     oid, _, _ = oid_with_credentials_authz
-    connection = KeycloakOpenIDConnection(
+    return KeycloakOpenIDConnection(
         server_url=oid.connection.base_url,
         realm_name=oid.realm_name,
         client_id=oid.client_id,
         client_secret_key=oid.client_secret_key,
         timeout=60,
     )
-    yield connection
 
 
 @pytest.fixture
-def uma(oid_connection_with_authz: KeycloakOpenIDConnection):
-    """Fixture for initialized KeycloakUMA class.
+def uma(oid_connection_with_authz: KeycloakOpenIDConnection) -> KeycloakUMA:
+    """
+    Fixture for initialized KeycloakUMA class.
 
     :param oid_connection_with_authz: Keycloak open id connection with pre-configured authz client
     :type oid_connection_with_authz: KeycloakOpenIDConnection
@@ -527,4 +647,4 @@ def uma(oid_connection_with_authz: KeycloakOpenIDConnection):
     """
     connection = oid_connection_with_authz
     # Return UMA
-    yield KeycloakUMA(connection=connection)
+    return KeycloakUMA(connection=connection)
